/*
 * Originally written r. simic
 * (c) 2011 Industrie It Pty Ltd
 * http://www.industrieit.com/
 *
 * IndustrieIT Pty Ltd licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * The software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.industrieit.jsf.stateless.impl;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.component.UIViewRoot;

public class SJSFStatePool {

	private static Map<String, LinkedList<UIViewRoot>> pool=new HashMap<String, LinkedList<UIViewRoot>>();

	private static final String INPOOL="SJSF_INPOOL";
	private static final int REISSUE_DELAY_MS=20;
	
	public static int dodgeCount=0;
	
	public static synchronized UIViewRoot get(String uri)
	{
		LinkedList<UIViewRoot> items=(LinkedList<UIViewRoot>) pool.get(uri);
		if (items==null)
		{
			items=new LinkedList<UIViewRoot>();
			pool.put(uri, items);
		}
		if (items.isEmpty())
		{
			return null;
		}
		UIViewRoot v= (UIViewRoot) items.removeFirst();
		long intime=(Long) v.getAttributes().get(INPOOL);
		if (System.currentTimeMillis()-intime<REISSUE_DELAY_MS)
		{
			items.addFirst(v);
			return null;
		}
		else
		{
			return v;
		}
	}
	
	public static synchronized void clearPool()
	{
		pool=new HashMap<String, LinkedList<UIViewRoot>>();
	}
	
	public static synchronized void cache(String uri, UIViewRoot v)
	{
		LinkedList<UIViewRoot> items=(LinkedList<UIViewRoot>) pool.get(uri);
		if (items==null)
		{
			items=new LinkedList<UIViewRoot>();
			pool.put(uri, items);
		}
		if (!items.contains(v))
		{
			if (v.getChildCount()==0)
			{
				dodgeCount++;
				System.out.println("ERRRRORRRRRR ON CREATE VIEW!!!!!!");
				return;
			}
			v.getViewMap().clear();
			v.getAttributes().put(INPOOL, System.currentTimeMillis());
			reset(v);
			items.addLast(v);
		}
	}
	
	public static int getPoolCount(String uri)
	{
		LinkedList<UIViewRoot> items=(LinkedList<UIViewRoot>) pool.get(uri);
		if (items==null)
		{
			return 0;
		}
		return items.size();
	}
	
	private static void reset(UIComponent c)
	{
		for (UIComponent child : c.getChildren())
		{
			if (child instanceof UIInput)
			{
				UIInput i=(UIInput) child;
				i.resetValue();
			}
			reset(child);
		}
	}	 
}
